
// Comp_To_UI.jsx
// version 1.3 by CR Green (www.crgreen.com)
// v1.3 adds rounding to avoid annoying floats
// A script that uses an After Effects comp as a user-interface mock-up to make a user interface for a script for After Effects!

// Documentation should have been included with this script.

//globals:
winVar = "win";
wVar = "w";
buildFuncVar = "buildUI";
diaOrPal = "dialog";

var optionWin = new Window('dialog', 'UI Generator Settings',[300,100,670,270]);

var optionsW = buildOptionsUI();

if (optionsW != null) {
   optionWin.styleDiaRad.value = true;
   optionsW.show();
}

function mainCodeMaker() {
   
   if (app.project != null) {
      if (app.project.activeItem != null) {
         if (app.project.activeItem instanceof CompItem) {
            // window position:
            winCompX = 300;
            winCompY = 100;
            //   "panel", "button", "radiobutton", "checkbox", "edittext", "statictext", "scrollbar", "slider", "image", "progressbar", "scrollbar"
            var controlClasses = "\rpanel\rbutton\rradiobutton\rcheckbox\redittext\rstatictext\rscrollbar\rslider\rimage\rprogressbar\rscrollbar\r";
            theComp = app.project.activeItem;
            wNa = theComp.name;
            wWidth = theComp.width;
            wHeight = theComp.height;
            
            theLayers = theComp.layers;
            theCode = "var " + winVar + " = new Window('" + diaOrPal + "', '" + wNa + "',[" + winCompX + "," + winCompY + "," + 
            (winCompX+wWidth) + "," + (winCompY+wHeight) + "]);\r" +
            "var " + wVar + " = " + buildFuncVar + "();\rif (" + wVar + " != null) {\r " + wVar + ".show();\r}\r\rfunction " + 
            buildFuncVar + "() {\r if (" + winVar + " != null) {\r";
                  usableLayerCount = 0;

                  for (o = theLayers.length; o >= 1; o--) {
                     thisLayer = theLayers[o];
                     // if there is no source, don't do a thing
                     
                     if (thisLayer.source != null) {
                        apX = thisLayer.anchorPoint.value[0];
                        apY = thisLayer.anchorPoint.value[1];
                        thisLayerW = thisLayer.width;
                        thisLayerH = thisLayer.height;
                        
                        thisLayerX = thisLayer.position.value[0];
                        thisLayerY = thisLayer.position.value[1];
                        
                        // if the anchor point is not 00
                        if ([apX, apY] != [0, 0]) {
                           apOS = (-1 * ([0, 0] - [apX, apY]));
                           posOS = -1 * apOS;
                           thisLayer.anchorPoint.setValue([0, 0]);
                           thisLayer.position.setValue([thisLayerX, thisLayerY] + posOS);
                           thisLayerX = thisLayer.position.value[0];
                           thisLayerY = thisLayer.position.value[1];
                        }
                        
						thisLayerX = (Math.round(thisLayerX));
						thisLayerY = (Math.round(thisLayerY));
                        x2 = Math.round((thisLayerX + thisLayerW));
                        y2 = Math.round((thisLayerY + thisLayerH));
                        
                        if (thisLayer.property("Marker").numKeys == 0) {
                           // if no markers
                           thisLayerCom = "";
                        } else {
                           //get the comment if there is one
                           thisLayerCom = thisLayer.property("Marker").keyValue(1).comment;
                        }
                        layerNa = thisLayer.name;
                        srcNa = thisLayer.source.name;
                        //first check for space:
                        spaceOS = srcNa.indexOf(" ");
                        if (spaceOS != -1) {
                           //a space! ew! get rid of it!
                           classNa = srcNa.substring(0, spaceOS);
                        } else {
                           classNa = srcNa;
                        }
                        //now see if it is a known class:
                        classOffset = controlClasses.indexOf( ("\r" + classNa + "\r") );
                        if (classOffset != -1) {
                           if (classNa == "button") {
                              //if it's a button, we include name and window-closing function
                              theCode = (theCode + " " + winVar + "." + layerNa + " = " + winVar + ".add('" + classNa + "', [" + thisLayerX + "," + 
                              thisLayerY + "," + x2 + "," + y2 + "], '" + thisLayerCom + "', {name:'" + thisLayerCom + "'});\r" + 
                              " " + winVar + "." + layerNa + ".onClick = function () {this.parent.close(1)};\r" );
                           } else if (classNa == "radiobutton") {
                              // if it's a radio button, we need to
                              //  (a) see if it has a parent
                              //  (b) get its parent
                              //  (c) use this construction: win.radioLayerName = win.parentLayerName.add etc.
                              //  (the position should take care of itself, since, if parented in the normal way, the position property is returned as relative
                              if (thisLayer.parent != null) {
                                 theCode = (theCode + " " + winVar + "." + layerNa + " = " + winVar + "." + (thisLayer.parent.name) + ".add('" + classNa + "', [" + thisLayerX + "," + 
                                 thisLayerY + "," + x2 + "," + y2 + "], '" + thisLayerCom + "');\r" );
                              } else {
                                 theCode = (theCode + " " + winVar + "." + layerNa + " = " + winVar + ".add('" + classNa + "', [" + thisLayerX + "," + 
                                 thisLayerY + "," + x2 + "," + y2 + "], '" + thisLayerCom + "');\r" );
                              }
                              
                           } else {
                              theCode = (theCode + " " + winVar + "." + layerNa + " = " + winVar + ".add('" + classNa + "', [" + thisLayerX + "," + 
                              thisLayerY + "," + x2 + "," + y2 + "], '" + thisLayerCom + "');\r" );
                           }
                        } else {
                           // not in known classes; treat in generic way:
                           theCode = (theCode + " " + winVar + "." + layerNa + " = " + winVar + ".add('" + classNa + "', [" + thisLayerX + "," + 
                           thisLayerY + "," + x2 + "," + y2 + "], '" + thisLayerCom + "');\r" );
                        }
                        usableLayerCount = (usableLayerCount + 1);
                     }// if (thisLayer.source != null)
                  }
            theCode = (theCode + "}\rreturn " + winVar + "\r}\r");
            if (usableLayerCount > 0) {
               codeLayer = theComp.layers.addText(theCode);
               codeLayer.name = "your code!";
               codeLayer.position.setValue([1, 12]);
            } else {
               alert("Not enough usable layers.");
            }
         }
      }
   }
}

function buildOptionsUI() {
   if (optionWin != null) {
      optionWin.stylePnl = optionWin.add('panel', [17,34,117,124], 'Style:');
      optionWin.varsPnl = optionWin.add('panel', [127,7,356,124], 'Variable Names:');
      optionWin.styleDiaRad = optionWin.stylePnl.add('radiobutton', [15,26,85,48], 'Dialog');
      optionWin.stylePalRad = optionWin.stylePnl.add('radiobutton', [15,57,85,79], 'Palette');
      optionWin.winVarLbl = optionWin.add('statictext', [137,32,249,54], 'window definition:');
      optionWin.winVarT = optionWin.add('edittext', [254,29,344,51], winVar);
      optionWin.builtWinVarLbl = optionWin.add('statictext', [137,62,249,84], 'built window:');
      optionWin.builtWinVarT = optionWin.add('edittext', [254,59,344,81], wVar);
      optionWin.mainFuncLbl = optionWin.add('statictext', [137,92,249,114], 'main function:');
      optionWin.mainFuncT = optionWin.add('edittext', [254,90,344,112], buildFuncVar);
      optionWin.okBtn = optionWin.add('button', [273,136,353,158], 'OK', {name:'OK'});
      optionWin.okBtn.onClick = function () {grabOptions();this.parent.close(1);mainCodeMaker();};
      optionWin.cancBtn = optionWin.add('button', [173,136,253,158], 'Cancel', {name:'Cancel'});
      optionWin.cancBtn.onClick = function () {this.parent.close(0)};
      optionWin.helpBtn = optionWin.add('button', [17,136,97,158], 'Help!', {name:'Help!'});
      optionWin.helpBtn.onClick = pomoc;
   }
   return optionWin
}

function grabOptions() {
   if (optionWin.styleDiaRad.value != true) {
      diaOrPal = "palette";
   }
   winVar = optionWin.winVarT.text;
   wVar = optionWin.builtWinVarT.text;
   buildFuncVar = optionWin.mainFuncT.text;
}

function pomoc() {
   alert("Comp_To_UI (Composition to User Interface)\rThis script is supposed to be used with a \"mock-up\" of a script window in After Effects. " +
   "A comp's size and name become those of the " + 
   "generated window. The comp's layers become the various control objects, like buttons, panels, text fields, etc.\r\rUse source names for " +
   "class names, layer names for property references, marker comments for values, and parenting for grouping radio buttons within panels." + 
   " The script will work from the bottom layer upward (which means panels should be at the bottom), and produces a text layer whose text " +
   "may be copied and pasted into a script and run " +
   "to create a window.\r\rPlease refer to the html documentation that should have been included.");
}
